
import java.rmi.*;
import java.util.ArrayList;
import java.rmi.server.UnicastRemoteObject;

/**
 * The Remote News Client that receives news from server via RMI
 * Implementing NewsClientI
 * @see NewsClientI
 * @see UnicastRemoteObject
 */
public class MyClient extends UnicastRemoteObject implements NewsClientI{

    /**
     * The server to which It'll be connected via RMI
     */
    NewsServerI server;

    /**
     * The RMI address of client
     */
    static private String clientRMI;

    /**
     * The RMI address of server
     */
    static private String serverRMI;

    /**
     * The name of the client
     */
    private String thename;

    /**
     * The ID of the client
     */
    int ID;

    /**
     * Number of Clients produced by factory
     */
    static int count = 0;

    /**
     * News list
     */
    static private ArrayList hitlist;

    /**
     * Enable client to receive news
     * @param newsItem desired news
     * @return for convention only
     * @throws RemoteException if link has broken
     */
    public int receiveNewsItem(String newsItem) throws RemoteException {
        hitlist.add(newsItem);
        return 0;
    }

    /**
     * The constructor
     * @param serverName the name of the server
     * @param name the name of the client
     * @throws RemoteException if link has broken
     */
    private MyClient(String serverName, String name) throws RemoteException {
        super();
        thename = name;

        try {
            server = (NewsServerI) Naming.lookup(serverRMI + serverName);
        } catch (NotBoundException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        }

        try {
            Naming.bind(clientRMI + name, this);
        } catch (AlreadyBoundException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        }

        try {
            ID = server.subscribe(clientRMI + thename);
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        }
    }

    /**
     * This methods is for test
     */
    public void go() {
        try {
            server.declareInterest(ID, "news");
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        }
    }

    /**
     * The factory method
     * @param name The RMI name
     * @param serverName The RMI name of the server
     * @param clientRMIp The RMI address to be bound
     * @param serverRMIp The RMI address of the server
     * @param hitlistp The list to which news will be sent
     * @return produced Client
     */
    static public MyClient getInstance(String name, String serverName, String clientRMIp,
                                       String serverRMIp, ArrayList hitlistp) {
        count++;

        if(count > 1)
            return null;

        clientRMI = clientRMIp;
        serverRMI = serverRMIp;
        hitlist = hitlistp;

        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        MyClient mc = null;

        try {
            mc = new MyClient(serverName, name);
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            System.out.println(e.getStackTrace());
        }

        return mc;
    }

    /**
     * Unbinding client from RMIregistry
     */
    public void end() {
        try {
            Naming.unbind(clientRMI + thename);
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (NotBoundException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        }

        count--;
    }

    /**
     * For test
     * @param args For test
     */
    public static void main(String[] args) {
        MyClient client = getInstance("MyClient", "MyServer", "//localhost:3131/", "//localhost:3131/", null);
        client.go();
    }
}
