
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * This class helps server to cantain its clients information
 * @see MyServer
 */
public class ClientID implements Serializable {

    /**
     * The client object that has earned through RMI
     */
    private NewsClientI client;

    /**
     * The client's ID
     */
    private int ID;

    /**
     * List of interested topics
     */
    private ArrayList interests;

    /**
     * Determine if this client is interested in this news
     * @param context the body of the news
     * @return if it interests
     */
    public boolean getInterests(String context) {
        StringTokenizer st = new StringTokenizer(context);

        while (st.hasMoreTokens()) {
            String next = st.nextToken();

            for (int i = 0; i < interests.size(); i++)
                if (interests.get(i).equals(next))
                    return true;
        }

        return false;
    }

    /**
     * Simple getter for ID
     * @return ID
     */
    public int getID() {
        return ID;
    }

    /**
     * Call the RMI method of client to receiving news
     * @param newsItem news
     */
    public void sendNews(String newsItem) {
        try {
            client.receiveNewsItem(newsItem);
        } catch (RemoteException e) {
            System.out.println("The RMI Connection to " + ID + " has been corrupted");
        }
    }

    /**
     * Add keyword to interests
     * @param interest interested keyword
     */
    public void addInterest(String interest) {
        interests.add(interest);
    }

    /**
     * The constructor
     * @param client The client
     * @param ID The ID
     */
    public ClientID(NewsClientI client, int ID) {
        this.client = client;
        this.ID = ID;
        interests = new ArrayList();
    }
}
