
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.rmi.*;
import java.sql.*;
import java.util.ArrayList;
import java.rmi.server.UnicastRemoteObject;

/**
 * The Remote News Server that sends news for clients via RMI
 * Implementing NewsServerI
 * @see NewsServerI
 * @see UnicastRemoteObject
 */
public class MyServer extends UnicastRemoteObject implements NewsServerI {

    /**
     * The address of the data base to connect
     */
    static private String serverAddress;

    /**
     * The name of the data base that will be connecteds
     */
    static private String dataBaseName;

    /**
     * The user name of the data base to be connected
     */
    static private String usrName;

    /**
     * The password of the data base
     */
    static private String pass;

    /**
     * RMI Address of server that this object will be bound
     */
    static private String serverRMI;

    /**
     * The RMI name to register
     */
    private String thename;

    /**
     * List of clients
     * @see ClientID
     */
    static private ArrayList users = new ArrayList();

    /**
     * The next ID for the client
     */
    private int freeID;

    /**
     * Number of clients produced using getInstance
     */
    static private int count = 0;

    /**
     * The news that has retrived from data base
     */
    private String[] result = null;

    /**
     * An RMI method for clients to subscribe and get ID
     * @param RMIAddress the RMI address of client
     * @return ID
     * @throws RemoteException if linke has problem
     */
    public int subscribe(String RMIAddress) throws RemoteException {
        freeID++;

        try {
            synchronized(users) {
                NewsClientI mc = (NewsClientI) Naming.lookup(RMIAddress);
                users.add(new ClientID(mc, freeID));
            }
        } catch (NotBoundException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        }

        return freeID;
    }

    /**
     * An RMI method for clients to add keyword to its interests
     * @param subscribeID the client's ID
     * @param keyword Interested keyword
     * @throws RemoteException if link has problem
     */
    public void declareInterest(int subscribeID, String keyword) throws RemoteException {
        for (int i = 0; i < users.size(); i++) {
            if (((ClientID) users.get(i)).getID() == subscribeID) {
                synchronized(users.get(i)) {
                    ((ClientID) users.get(i)).addInterest(keyword);
                }

                return;
            }
        }

        System.out.println("Desired ID does not exist !!");
    }

    /**
     * The constructor
     * @param name the binding name
     * @throws RemoteException from its super class
     */
    private MyServer(String name) throws RemoteException {
        super();
        thename = name;
        freeID = 0;
    }

    /**
     * Sends news to all of the clients
     */
    public void broadcastNews() {
        synchronized(users){
           for(int j = 0; j < users.size(); j++) {
                synchronized(result){
                    for (int i = 0; i < result.length; i++) {
                        if (((ClientID)users.get(j)).getInterests(result[i]))
                            ((ClientID)users.get(j)).sendNews(result[i]);
                    }
                }
            }
        }
    }

    /**
     * Loads news from XML file
     */
    private void getXMLNews() {
        ArrayList news = new ArrayList();

        DefaultHandler handler= new SAXHandler(news);
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(true);

        try {
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse("file.xml", handler);
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        String[] myresult = new String[news.size()];

        for (int i = 0; i < news.size(); i++)
            myresult[i] = (String) news.get(i);

        result = myresult;
    }

    /**
     * Connects to data base and retrive all the news
     */
    private void getNews() {
        try {
            DriverManager.registerDriver(new com.microsoft.jdbc.sqlserver.SQLServerDriver());
            Connection connection = DriverManager.getConnection("jdbc:microsoft:sqlserver://" + serverAddress + "DatabaseName=" + dataBaseName, usrName, pass);
            Statement select = connection.createStatement();
            String query = "select * from news";
            ResultSet rs = select.executeQuery(query);

            ArrayList results = new ArrayList();

            while (rs.next())
                results.add(rs.getString("newsBody"));

            String[] myresult = new String[results.size()];

            for (int i = 0; i < results.size(); i++)
                myresult[i] = (String) results.get(i);

            connection.close();
            select.close();
            rs.close();

            result = myresult;
        } catch (SQLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
            System.out.println("Cannot Stablish Connection to Data Base");
        }
    }

    /**
     * Unbind server from RMI registry
     */
    public void end() {
        try {
            Naming.unbind(serverRMI + thename);
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        } catch (NotBoundException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        }

        count--;
    }

    /**
     * Factory method of the class
     * @param name The name of the server
     * @param serverAddressp The data base address
     * @param dataBaseNamep The name of the data base
     * @param usrNamep The user name of the data base
     * @param passp The password of the data base
     * @param serverRMIp The RMI address to bind
     * @return Produced server
     */
    static public MyServer getInstance(String name, String serverAddressp, String dataBaseNamep,
                                        String usrNamep, String passp, String serverRMIp) {
        count++;

        if(count > 1)
            return null;

        serverAddress = serverAddressp;
        dataBaseName = dataBaseNamep;
        usrName = usrNamep;
        pass = passp;
        serverRMI = serverRMIp;

        MyServer ms = null;

        try {
            ms = new MyServer(name);
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            System.out.println(e.getStackTrace());
        }

        ms.getNews();
//        ms.getXMLNews();

        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        try {
            Naming.bind(serverRMI + name, ms);
        } catch (AlreadyBoundException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (java.net.MalformedURLException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        } catch (RemoteException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();
        }

        return ms;
    }

    /**
     * For test
     * @param args for test
     */
    public static void main(String[] args) {
        MyServer server = getInstance("MyServer", "192.168.98.40:1433;", "DBLABS8302", "DBLABS8302", "kami", "//localhost:3131/");
        System.out.println("Enter: ");

        try {
            System.in.read();
        } catch (IOException e) {
            System.out.println(e.getMessage());
            e.getStackTrace();

        }

        server.end();
    }
}
